﻿#include <iostream>
#include <vector>
#include <iomanip>
#include <algorithm>

const size_t SCREEN_WIDTH = 80; // Максимальная ширина экрана
const size_t MAX_ASTERISK = SCREEN_WIDTH - 4; // Ограничение на длину столбца

// Функция для ввода данных: количество чисел и сами числа
void input_data(std::vector<double>& numbers, size_t& bin_count) {
    size_t number_count;
    std::cerr << "Enter number count: ";
    std::cin >> number_count;

    numbers.resize(number_count);
    std::cerr << "Enter numbers: ";
    for (size_t i = 0; i < number_count; ++i) {
        std::cin >> numbers[i];
    }

    std::cerr << "Enter bin count: ";
    std::cin >> bin_count;
}

// Функция для расчета количества чисел в каждой корзине
void calculate_bins(const std::vector<double>& numbers, std::vector<size_t>& bins, double& min, double& max) {
    // Определяем минимальное и максимальное значение для диапазона
    min = *std::min_element(numbers.begin(), numbers.end());
    max = *std::max_element(numbers.begin(), numbers.end());

    // Вычисляем размер каждой корзины
    double bin_size = (max - min) / bins.size();

    // Заполняем корзины, увеличивая счётчик для каждого числа, попадающего в интервал
    for (const double& number : numbers) {
        size_t bin_index = static_cast<size_t>((number - min) / bin_size);
        if (bin_index >= bins.size()) bin_index = bins.size() - 1;
        bins[bin_index]++;
    }
}

// Функция для вывода гистограммы
void print_histogram(const std::vector<size_t>& bins) {
    // Находим максимальное количество чисел в одной корзине для масштабирования
    size_t max_count = *std::max_element(bins.begin(), bins.end());

    // Проверка необходимости масштабирования
    bool needs_scaling = max_count > MAX_ASTERISK;

    for (size_t i = 0; i < bins.size(); ++i) {
        std::cout << std::setw(3) << std::right << bins[i] << "|";

        // Рассчитываем высоту столбца с учётом ограничения MAX_ASTERISK
        size_t height = needs_scaling ? static_cast<size_t>(MAX_ASTERISK * static_cast<double>(bins[i]) / max_count) : bins[i];

        // Выводим звёздочки для визуализации количества элементов
        for (size_t j = 0; j < height; ++j) {
            std::cout << "*";
        }
        std::cout << "\n";
    }
}

int main() {
    std::vector<double> numbers; // Массив чисел
    size_t bin_count; // Количество корзин

    input_data(numbers, bin_count); // Ввод данных

    std::vector<size_t> bins(bin_count, 0); // Инициализируем корзины с нулями
    double min, max;
    calculate_bins(numbers, bins, min, max); // Подсчёт чисел в корзинах
    print_histogram(bins); // Вывод гистограммы

    return 0;
}
